app.controller('gmapCtrl', ['$scope', '$rootScope', '$log', '$tm1Ui', '$ngBootbox','screenSize','$timeout', '$location', function ($scope, $rootScope, $log, $tm1Ui, $ngBootbox, screenSize, $timeout, $location) {
    ////Sorting

    $scope.sortBy = function (key) {

        if ($scope.sortPeriod === key) {
            $scope.sortDirection = $scope.sortDirection === 'BDESC' ? 'BASC' : 'BDESC';
        } else {
            $scope.sortPeriod = key;
        }

    };

    $scope.sortBy1 = function (key) {

        if ($scope.sortPeriod1 === key) {
            $scope.sortDirection1 = $scope.sortDirection1 === 'BDESC' ? 'BASC' : 'BDESC';
        } else {
            $scope.sortPeriod1 = key;
        }

    };

    $scope.drillDown = function (key) {
        $scope.productDrill = key;
    };
    /*
     *     defaults.* are variables that are declared once and are changed in the page, otherwise known as constants in programming languages
     *     lists.* should be used to store any lists that are used with ng-repeat, i.e. tm1-ui-element-list
     *     selections.* should be used for all selections that are made by a user in the page
     *     values.* should store the result of any dbr, dbra or other values from server that you want to store to use elsewhere, i.e. in a calculation
     * 
     *     For more information: https://github.com/cubewise-code/canvas-best-practice
     */

    $scope.lists = {};
    $scope.values = {};

    $scope.screenSize = screenSize;
    

    // Default values to be used before selections are made
    $scope.defaults = {
        region: 'Total Europe',
        department: 'Corporate',
        measure: 'Order Quantity',
        version: 'Actual',
        product: 'Accessories',
        month: 'Year',
        year: '2016',
        OperatingProfit: 'Operating Profit',
        GrossMargin: 'Gross Margin',
        OperatingExpenses: 'Operating Expenses',
        OtherIncome: 'Other Income and Expense'
    }

    $scope.selections = {
        showContent: 'table',
        month: $scope.defaults.month,
        year: $scope.defaults.year,
        version: $scope.defaults.version,
        region: $scope.defaults.region,
        department: $scope.defaults.department,
        product: $scope.defaults.product,
        measure: $scope.defaults.measure,
        segment: 'region',
        subset: [
            {
                description: "Operating Profit",
                icon: ""
            },
            {
                description: "Gross Margin"
            },
            {
                description: "Operating Expenses"
            },
            {
                description: "Other Income and Expense"
            }
		]
    };

    // ###############
    // Update Product parameter

    if($location.search().product){
        $scope.selections.product = $location.search().product;
    }

    $scope.setProduct = function (product) {
        $scope.selections.product = product;
        if (product != $scope.defaults.product) {
            // Set the URL parameter
            $location.search("product", product);
        } else {
            // If the parameter equals the default value remove it from the URL
            $location.search("product", null);
        }
    };

    // ###############
    // Update Region parameter

        if($location.search().region){
        $scope.selections.region = $location.search().region;
    }

    $scope.setRegion = function (region) {
        $scope.selections.region = region;
        if (region != $scope.defaults.region) {
            // Set the URL parameter
            $location.search("region", region);
        } else {
            // If the parameter equals the default value remove it from the URL
            $location.search("region", null);
        }
    };

    // ##################
    // Clickablke chart to update region
    $scope.optionsOverrideRegion = {
        chart: {
            discretebar: {
                dispatch: {
                    
                    elementClick: function (e) {
                        $scope.selections.region = e.data.elements.split(',')[4];
                    }
                }
            }

        }
    };

    if(screenSize.is('xs, sm')){
	$scope.optionsOverrideRegion.chart.discretebar.dispatch.elementClick = undefined;
		$scope.optionsOverrideRegion.chart.discretebar.dispatch.elementMouseover = function(e){ 
			console.debug('elementMouseover');
            $scope.setRegion(e.data.elements.split(',')[4]);
			//$scope.selections.region = e.data.elements.split(',')[4];
		};
    }


    // ############################
    //         Google Map - Function to flat TM1 data

    function tm1flat(tm1o, attrs, bFilter, inclAttrs) {
        //function to convert array into object
        function toObject(val, label) {
            var obj = {};
            for (var i = 0; i < label.length; i++)
                if (val[i] !== undefined && label[i] !== undefined) obj[label[i]] = val[i];
            return obj;
        }
        //========================================================

        //collect all dimensions from rows and columns
        //1) dimensions on columns
        var dims = [];

        for (var i = 0; i < tm1o.Axes[1].Hierarchies.length; i++) {
            dims.push(tm1o.Axes[1].Hierarchies[i].Name);
        }
        //2)dimensions on rows 
        for (var i = 0; i < tm1o.Axes[0].Hierarchies.length; i++) {
            dims.push(tm1o.Axes[0].Hierarchies[i].Name);
        }

        //3)OPTIONAL.filters
        if (bFilter != null && bFilter == true) {
            for (var i = 0; i < tm1o.Axes[2].Hierarchies.length; i++) {
                dims.push(tm1o.Axes[2].Hierarchies[i].Name);
            }
        }

        //4)add value to the list of dimensions
        dims.push('TM1CubeValue');

        if (inclAttrs) {
            var attributes = {};
        }


        //create auxilary arrays. rc is the main one. it contains all reference points + value as the last element.tm1fo is the array to be returned.
        var r1 = [],
            c1 = [],
            rc = [];
        var cellCount = 0;
        var tm1fo = [];

        //loop through rows
        for (var j = 0; j < tm1o.Axes[1].Tuples.length; j++) {
            for (var j1 = 0; j1 < tm1o.Axes[1].Tuples[j].Members.length; j1++) {

                if (attrs != null && tm1o.Axes[1].Tuples[j].Members[j1].Attributes[attrs[tm1o.Axes[1].Hierarchies[j1].Name]] != null) {
                    r1.push(tm1o.Axes[1].Tuples[j].Members[j1].Attributes[attrs[tm1o.Axes[1].Hierarchies[j1].Name]]);
                } else {
                    r1.push(tm1o.Axes[1].Tuples[j].Members[j1].Name);
                }


                //add dimensions attributes
                if (inclAttrs) {

                    attributes[tm1o.Axes[1].Hierarchies[j1].Name] = {};

                    for (var key in tm1o.Axes[1].Tuples[j].Members[j1].Attributes) {
                        attributes[tm1o.Axes[1].Hierarchies[j1].Name][key] = tm1o.Axes[1].Tuples[j].Members[j1].Attributes[key];
                    }
                }


            }
            //loop through columns
            for (var k = 0; k < tm1o.Axes[0].Tuples.length; k++) {
                for (var k1 = 0; k1 < tm1o.Axes[0].Tuples[k].Members.length; k1++) {
                    if (attrs != null && tm1o.Axes[0].Tuples[k].Members[k1].Attributes[attrs[tm1o.Axes[0].Hierarchies[k1].Name]] != null) {
                        c1.push(tm1o.Axes[0].Tuples[k].Members[k1].Attributes[attrs[tm1o.Axes[0].Hierarchies[k1].Name]]);
                    } else {
                        c1.push(tm1o.Axes[0].Tuples[k].Members[k1].Name);
                    }

                    if (inclAttrs) {
                        //add dimensions attributes
                        attributes[tm1o.Axes[0].Hierarchies[k1].Name] = {};

                        for (var key in tm1o.Axes[0].Tuples[k].Members[k1].Attributes) {
                            attributes[tm1o.Axes[0].Hierarchies[k1].Name][key] = tm1o.Axes[0].Tuples[k].Members[k1].Attributes[key];
                        }
                    }


                }
                rc = r1.concat(c1);

                //include filter values
                if (bFilter != null && bFilter == true) {
                    for (var m = 0; m < tm1o.Axes[2].Tuples[0].Members.length; m++) {
                        if (attrs != null && tm1o.Axes[2].Tuples[0].Members[m].Attributes[attrs[tm1o.Axes[2].Hierarchies[m].Name]] != null) {
                            rc.push(tm1o.Axes[2].Tuples[0].Members[m].Attributes[attrs[tm1o.Axes[2].Hierarchies[m].Name]]);
                        } else {
                            rc.push(tm1o.Axes[2].Tuples[0].Members[m].Name);
                        }
                    }
                }
                //add value from Cells
                rc.push(tm1o.Cells[cellCount].Value);
                //console.log(rc);

                tm1fo.push(toObject(rc, dims));

                if (inclAttrs) {
                    tm1fo[cellCount].Attributes = JSON.parse(JSON.stringify(attributes));
                }


                cellCount += 1;

                //zero out auxilary arrays
                c1 = [];
                rc = [];
            }
            r1 = [];
            if (inclAttrs) {
                attributes = {};
            }


        }

        //console.log(tm1fo);
        return tm1fo;
    };
//map


var map;


    var icons = {
        neutral: '',
        positive: 'images/'+$scope.selections.product+'.png',
        negative: 'images/red75.png'
    };
    var markers = [];
    

    $scope.initMap = function (){
        map = new google.maps.Map(document.getElementById('map'), {
            center: new google.maps.LatLng(48.8, 2.35),
            zoom: 4
        });
        // Vincent Hide componenents !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
    var styleArray = [
    {
        featureType: "all",
        stylers: [
        { visibility: "on" },
        { saturation: -90 }
        ]
    },
    // ROADS
    {
        featureType: "road",
        stylers: [
        { visibility: "off" }
        ]
    },
    // Makers / Monuments
    {
        featureType: "poi",
        elementType: "labels",
        stylers: [
        { visibility: "off" }
        ]
    },
    // Transit
    {
        featureType: "transit",
        elementType: "labels",
        stylers: [
        { visibility: "off" }
        ]
    }
    ];

    map.setOptions({styles: styleArray});
        // End Vincent ==============================================================
        
    };

    $scope.reCenter = function (address, lat, lon) {

        map.setCenter({
            lat: Number(lat),
            lng: Number(lon)
        });

        map.setZoom(15);

    };

    function removeMarkers() {
        for (var i = 0; i < markers.length; i++) {
            markers[i].setMap(null);
        }
        markers.length = 0;
    }


    function heatPoint(lat, lon, value, min, max, address) {

        var marker = new google.maps.Marker({
            position: {
                lat: lat,
                lng: lon
            },
            map: map,
            opacity: 1,
            icon: (value > 0 ? icons.positive : icons.negative)
        });
        
        markers.push(marker);
        
        marker.addListener('click', function () {
            var mdx = 'select {[Retail Measure].['+$scope.selections.measure+']} on 0, {[Store].[' + address + ']} on 1 from [Retail by Store] where ([Version].['+$scope.selections.version+'],[Year].['+$scope.selections.year+'],[Period].[Year],[Currency].[local],[Region].['+$scope.selections.region+'],[Product].['+$scope.selections.product+'])';
            $tm1Ui.cubeExecuteMdx('dev', mdx).then(function (data) {
                
                $scope.customers = tm1flat(data, {}, false, true);

                var content = '<div>' + address + '<ul>';

                for (var i = 0; i < $scope.customers.length; i++) {
                    if($scope.selections.measure == 'Order Quantity'){
                    content = content + '<li>' + $scope.selections.measure + ': ' + accounting.formatNumber($scope.customers[i]['TM1CubeValue']) + '</li>';
                    }else{
                    content = content + '<li>' + $scope.selections.measure + ': €' + accounting.formatMoney($scope.customers[i]['TM1CubeValue'], "€", 2, ".", ",") + '</li>';   
                    }
                }
                content = content + '</ul></div>';

                var infowindow = new google.maps.InfoWindow({
                    content: content
                });

                infowindow.open(map, marker);

            });
        });

    }

    $scope.mainQuery = function() {


        icons.positive = 'images/'+$scope.selections.product+'.png';
        icons.negative = 'images/red75.png';
        
        removeMarkers();
        var mdx = "select {[Retail Measure].["+$scope.selections.measure+"]} on 0, NON EMPTY {TM1FILTERBYLEVEL( {TM1SUBSETALL( [Store] )}, 0)} on 1 from [Retail by Store] where ([Version].["+$scope.selections.version+"],[Year].["+$scope.selections.year+"],[Period].[Year],[Currency].[local],[Region].["+$scope.selections.region+"],[Product].["+$scope.selections.product+"])";
        $tm1Ui.cubeExecuteMdx('dev', mdx).then(function (data) {
            //console.log(data);

            var flatArray = tm1flat(data, {}, true, true);        
            $scope.data = flatArray;
            var min = d3.min(flatArray, function (d) {
                return d.TM1CubeValue;
            });
            
            var max = d3.max(flatArray, function (d) {
                return d.TM1CubeValue;
            });
 
            for (var i = 0; i < flatArray.length; i++) {
                //console.log(flatArray[i]);
                heatPoint(Number(flatArray[i]['Attributes']['Store']['Lat']), Number(flatArray[i]['Attributes']['Store']['Lon']), flatArray[i]['TM1CubeValue'], min, max, flatArray[i]['Attributes']['Store']['Description']);
            }

        });
    };

    $scope.initMap();

    $scope.showMap = function() {
        //if(!$scope.mapCreated){
            $timeout(function(){
        //        $scope.mapCreated = true;
                $scope.initMap();
                $scope.mainQuery();
            }, 50);
      //  }
    };
$scope.showMap();

  
    


    //map end

    function loopObject(o) {
        var temp;
        var tempName;
        
        for (var i = 0; i < values.length; i++) {
            
            if (values[i].Product == o.Name && values[i].Version == 'Var %') {
                
                temp=values[i].TM1CubeValue==null?0:values[i].TM1CubeValue;
                break;
            }
        }
        //console.log(o.Name+': '+temp)
        o.variance=temp;
        
        if (o.hasOwnProperty('Components')) {

            for (var i = 0; i < o.Components.length; i++) {
                loopObject(o.Components[i]);
            }

        } else {

            for (var i = 0; i < values.length; i++) {

                if (values[i].Product == o.Name && values[i].Version == $scope.selections.version) {
                    o.value = values[i].TM1CubeValue;
                    break;
                }
            }
        }
    }


 



    var values;
    var products;





    //var mdx = '{TM1DRILLDOWNMEMBER( {[product].[All Products by Category]}, ALL, RECURSIVE )}';
    var mdx = '[Product].MEMBERS';
    $tm1Ui.dimensionElements('dev', 'Product', 'Code&Description', '', mdx, '', '', true, false).then(function (v) {
        //console.log(v);
        products = v[0];
       // width = parseInt((document.getElementById("chart-one").parentNode.getBoundingClientRect().width)-(30));
        ///////////////////////////
          
    })
    
    
            
    $(window).resize(function(){
         //width = parseInt((document.getElementById("chart-one").parentNode.getBoundingClientRect().width)-(30));
         console.log(width);
          
    });
}]);
